"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var base_user_service_1 = require('./base-user.service');
/**
 * BaseRequestService helps to generate request api : GET - POST - DELETE
 * <br>
 * <strong>Usage</strong>
 * ```
 * import { Injectable } from '@angular/core';
 * import { BaseRequestService } from 'donkeycode-angular2-tools/src/services/base-request.service';
 * import { Http } from '@angular/http';
 * import { contentHeaders } from './headers';
 * import { config } from '../../config';
 *
 * @Injectable()
 * export class RequestService extends BaseRequestService {
 *
 *   constructor(public http: Http) {
 *     super(http);
 *     super.init(config, contentHeaders);
 *   }
 *
 * }
 * ```
 */
var BaseRequestService = (function () {
    function BaseRequestService(http) {
        this.http = http;
        this.activeRequest = null;
        this.config = {};
        this.contentHeaders = {};
    }
    BaseRequestService.prototype.init = function (config, contentHeaders) {
        this.config = config;
        this.contentHeaders = contentHeaders;
    };
    BaseRequestService.prototype.transformUrl = function (url) {
        if (url.indexOf('://') === -1) {
            url = this.config.baseApi + url;
        }
        if (url.indexOf('access_token') > -1) {
            return url;
        }
        var accessToken = localStorage.getItem('auth_access_token');
        if (accessToken) {
            var separator = '?';
            if (url.indexOf('?') > -1) {
                separator = '&';
            }
            url += separator + "access_token=" + accessToken;
        }
        return url;
    };
    BaseRequestService.prototype.parseMessage = function (response) {
        if (response && response._body) {
            return JSON.parse(response._body);
        }
        return response;
    };
    BaseRequestService.prototype.handleUnauthorized = function (resolve, reject, self, method, callbackArgs) {
        var _this = this;
        if (!localStorage.getItem('auth_refresh_token')) {
            resolve('no_refresh_token');
            base_user_service_1.BaseUserService.redirectLogin();
            return;
        }
        this.refreshToken()
            .then(function () {
            self[method](callbackArgs)
                .then(function (response) {
                resolve(_this.parseMessage(response));
            }, function (error) {
                reject(_this.parseMessage(error));
            });
        }, function (error) {
            resolve(_this.parseMessage(error));
            base_user_service_1.BaseUserService.redirectLogin();
        });
    };
    BaseRequestService.prototype.handleRequest = function (self, method, args, originalArgs) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.activeRequest = (_a = self['http'])[method].apply(_a, args)
                .subscribe(function (response) {
                resolve(_this.parseMessage(response));
            }, function (error) {
                if (error.status === 401) {
                    _this.handleUnauthorized(resolve, reject, self, method, originalArgs);
                }
                else {
                    reject(_this.parseMessage(error));
                }
            });
            var _a;
        });
    };
    BaseRequestService.prototype.post = function (url, body) {
        if (body === void 0) { body = {}; }
        var args = [this.transformUrl(url), JSON.stringify(body), { headers: this.contentHeaders }];
        var originalArgs = [url, body];
        return this.handleRequest(this, 'post', args, originalArgs);
    };
    BaseRequestService.prototype.put = function (url, body) {
        if (body === void 0) { body = {}; }
        var args = [this.transformUrl(url), JSON.stringify(body), { headers: this.contentHeaders }];
        var originalArgs = [url, body];
        return this.handleRequest(this, 'put', args, originalArgs);
    };
    BaseRequestService.prototype.get = function (url) {
        var args = [this.transformUrl(url), { headers: this.contentHeaders }];
        var originalArgs = [url];
        return this.handleRequest(this, 'get', args, originalArgs);
    };
    BaseRequestService.prototype.delete = function (url) {
        var args = [this.transformUrl(url), { headers: this.contentHeaders }];
        var originalArgs = [url];
        return this.handleRequest(this, 'delete', args, originalArgs);
    };
    BaseRequestService.prototype.refresh = function () {
        var body = {
            client_id: this.config.client_id,
            client_secret: this.config.client_secret,
            refresh_token: localStorage.getItem('auth_refresh_token'),
            grant_type: 'refresh_token'
        };
        return this.post('/noauth/oauth/v2/token', body);
    };
    BaseRequestService.prototype.refreshToken = function () {
        if (BaseRequestService.isRefreshingToken) {
            return BaseRequestService.isRefreshingToken;
        }
        BaseRequestService.isRefreshingToken = this.refresh()
            .then(function (response) {
            localStorage.setItem('auth_access_token', response.access_token);
            localStorage.setItem('auth_expires_in', response.expires_in);
            localStorage.setItem('auth_refresh_token', response.refresh_token);
            localStorage.setItem('auth_scope', response.scope);
            localStorage.setItem('auth_token_type', response.token_type);
            BaseRequestService.isRefreshingToken = null;
        });
        return BaseRequestService.isRefreshingToken;
    };
    BaseRequestService.prototype.getPaginatedUrl = function (url, page, max) {
        if (page === 0 && max === 0) {
            return url;
        }
        var separator = '?';
        if (url.indexOf('?') > -1) {
            separator = '&';
        }
        url += separator + "page=" + page + "&page_size=" + max;
        return url;
    };
    BaseRequestService.prototype.stopRequest = function () {
        if (this.activeRequest) {
            this.activeRequest.unsubscribe();
        }
        this.activeRequest = null;
    };
    BaseRequestService.isRefreshingToken = null;
    BaseRequestService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [http_1.Http])
    ], BaseRequestService);
    return BaseRequestService;
}());
exports.BaseRequestService = BaseRequestService;
//# sourceMappingURL=base-request.service.js.map